<?php
/**
 * @file
 * Drush integration to enable/disable development modules all at once.
 */

/**
 * Returns default list of development modules.
 *
 * @return array
 */
function dev_modules_dev_defaults() {
  return array(
    'browser_refresh',
    'coffee',
    'devel',
    'field_ui',
    'hacked',
    'link_css',
    'rules_ui',
    'stage_file_proxy',
    'views_ui',
  );
}

/**
 * Returns default list of production modules.
 *
 * @return array
 */
function dev_modules_production_defaults() {
  return array(
    'advagg',
    'bigpipe',
    'dynamic_page_cache',
    'page_cache',
    'purge',
    'varnish_purger',
  );
}

/**
 * Implements hook_drush_help().
 */
function dev_modules_drush_help($section) {
  switch ($section) {
    case 'drush:dev-modules':
      return dt('Enable/disable development and production modules all at once.');
  }
}

/**
 * Implements hook_drush_command().
 */
function dev_modules_drush_command() {
  $items = array();

  $items['dev-modules'] = array(
    'description' => 'Enable/disable development and production modules all at once.',
    'callback' => 'drush_dev_modules',
    'bootstrap' => DRUSH_BOOTSTRAP_DRUPAL_FULL,
    'arguments' => array(
      'flag' => 'On or off, yes or no, True or False, 1 or 0',
    ),
    'options' => array(
      'list' => 'Only determine the list of modules and print that list to the console',
    ),
    'aliases' => array('dev'),
  );

  return $items;
}

/**
 * Drush callback to execute the drush dev-modules command.
 *
 * @param $flag
 */
function drush_dev_modules($flag) {
  switch (strtolower($flag)) {
    case 'on':
    case 'yes':
    case 'true':
    case '1':
      $enable = TRUE;
      break;

    case 'off':
    case 'no':
    case 'false':
    case '0':
    $enable = FALSE;
      break;

    default:
      drush_set_error('Unsupported flag.');
      return;
  }

  // Collect all dev and production modules from the current installation and
  // allow other modules to alter the lists
  $dev_modules = dev_modules_dev_defaults();
  $production_modules = dev_modules_production_defaults();
  if (drush_drupal_major_version() < 8) {
    $dev_modules += module_invoke_all('dev_modules_dev');
    $production_modules += module_invoke_all('dev_modules_production');
    drupal_alter('dev_modules_dev', $dev_modules);
    drupal_alter('dev_modules_production', $production_modules);
  }
  else {
    $dev_modules += \Drupal::moduleHandler()->invokeAll('dev_modules_dev');
    $production_modules += \Drupal::moduleHandler()->invokeAll('dev_modules_production');
    \Drupal::moduleHandler()->alter('dev_modules_dev', $dev_modules);
    \Drupal::moduleHandler()->alter('dev_modules_production', $production_modules);
  }

  # TODO: Find dependencies.

  // Get the setting for this option
  $debug = drush_get_option('list');

  // Enable or disable all found dev modules
  if ($enable) {
    $list_enable = _dev_modules_enable($dev_modules, $debug);
    $list_disable = _dev_modules_disable($production_modules, $debug);
  }
  else {
    $list_disable = _dev_modules_disable($dev_modules, $debug);
    $list_enable = _dev_modules_enable($production_modules, $debug);
  }

  // Output the effective list
  if ($debug) {
    drush_print('Modules to disable: ' . implode(', ', $list_disable));
    drush_print('Modules to enable: ' . implode(', ', $list_enable));
  }
}

/**
 * Callback to enable all dev modules. This will take care that only those that
 * are really available in the code tree will actually be enabled.
 *
 * @param array $modules
 * @param bool $debug
 * @return array
 */
function _dev_modules_enable($modules, $debug) {
  $module_data = system_rebuild_module_data();
  $list = array();
  foreach ($modules as $module) {
    if (isset($module_data[$module])) {
      $list[] = $module;
    }
  }
  if (!$debug) {
    if (drush_drupal_major_version() < 8) {
      module_enable($list);
    }
    else {
      \Drupal::service('module_installer')->install($list, TRUE);
    }
  }
  return $list;
}

/**
 * Callback to disable all dev modules. This will take care that only those that
 * are currently enabled will actually be disabled.
 *
 * @param array $modules
 * @param bool $debug
 * @return array
 */
function _dev_modules_disable($modules, $debug) {
  $list = array();
  foreach ($modules as $module) {
    if ((drush_drupal_major_version() < 8 && module_exists($module)) ||
        (drush_drupal_major_version() >= 8 && \Drupal::moduleHandler()->moduleExists($module))) {
      $list[] = $module;
    }
  }
  if (!$debug) {
    if (drush_drupal_major_version() < 8) {
      module_disable($list);
    }
    else {
      \Drupal::service('module_installer')->uninstall($list);
    }
  }
  return $list;
}
